/*
 * Copyright (c) 2013, Victor Nazarov <asviraspossible@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Victor Nazarov nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.github.sviperll.collection;

import java.util.List;
import com.github.sviperll.io.Closeables;
import java.io.Closeable;
import java.util.Iterator;
import com.github.sviperll.Function;
import com.github.sviperll.Predicate;
import com.github.sviperll.collection.iterator.IteratorCloseableFluentIterator;

public class CloseableIterators {
    public static <T> CloseableFluentIterator<T> createClosableIterator(Iterator<T> iterator, Closeable closable) {
        return new IteratorCloseableFluentIterator<T>(iterator, closable);
    }

    public static <T, U extends Iterator<? extends T> & Closeable> CloseableFluentIterator<T> append(U iterator1, U iterator2) {
        return createClosableIterator(Iterators.append(iterator1, iterator2), Closeables.combine(iterator1, iterator2));
    }

    public static <T, U extends Iterator<? extends T> & Closeable> CloseableFluentIterator<T> append(U iterator1, U iterator2, U iterator3) {
        return createClosableIterator(Iterators.append(iterator1, iterator2), Closeables.combine(iterator1, iterator2));
    }

    public static <T, U extends Iterator<? extends T> & Closeable> CloseableFluentIterator<T> join(Iterator<U> iterators) {
        List<U> list = Iterators.decorate(iterators).intoNewArrayList();
        return createClosableIterator(Iterators.join(list.iterator()), Closeables.join(list));
    }

    public static <R, T, U extends Iterator<T> & Closeable> CloseableFluentIterator<R> map(U source, Function<? super T, ? extends R> function) {
        return createClosableIterator(Iterators.map(source, function), source);
    }

    public static <T, U extends Iterator<T> & Closeable> CloseableFluentIterator<T> filter(U source, Predicate<? super T> predicate) {
        return createClosableIterator(Iterators.filter(source, predicate), source);
    }

    public static <T, U extends Iterator<T> & Closeable> CloseableFluentIterator<T> take(U source, int count) {
        return createClosableIterator(Iterators.take(source, count), source);
    }

    public static <T, U extends Iterator<T> & Closeable> CloseableFluentIterator<T> drop(U source, int count) {
        return createClosableIterator(Iterators.drop(source, count), source);
    }

    private CloseableIterators() {
    }
}
