/*
 * Copyright (c) 2013, Victor Nazarov <asviraspossible@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Victor Nazarov nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.github.sviperll.collection.iterable;

import com.github.sviperll.BiFunction;
import java.util.List;
import java.util.Set;
import com.github.sviperll.collection.FluentIterable;
import com.github.sviperll.collection.Iterables;
import com.github.sviperll.collection.FluentIterator;
import com.github.sviperll.collection.Iterators;
import com.github.sviperll.Function;
import com.github.sviperll.Predicate;

public class IterableFluentIterable<T> implements FluentIterable<T> {
    private final Iterable<T> iterable;

    public IterableFluentIterable(Iterable<T> iterable) {
        this.iterable = iterable;
    }

    @Override
    public FluentIterator<T> iterator() {
        return Iterators.decorate(iterable.iterator());
    }

    @Override
    public FluentIterable<T> append(Iterable<? extends T> iterable) {
        return Iterables.append(this, iterable);
    }

    @Override
    public <R> FluentIterable<R> map(Function<? super T, ? extends R> function) {
        return Iterables.map(this, function);
    }

    @Override
    public FluentIterable<T> filter(Predicate<? super T> predicate) {
        return Iterables.filter(this, predicate);
    }

    @Override
    public FluentIterable<T> take(int count) {
        return Iterables.take(this, count);
    }

    @Override
    public FluentIterable<T> drop(int count) {
        return Iterables.drop(this, count);
    }

    @Override
    public <F> F fold(F seed, BiFunction<? super T, F, F> combiner) {
        return Iterables.fold(this, seed, combiner);
    }

    @Override
    public List<T> intoNewArrayList() {
        return Iterables.intoNewArrayList(this);
    }

    @Override
    public Set<T> intoNewTreeSet() {
        return Iterables.intoNewTreeSet(this);
    }

    @Override
    public Set<T> intoNewHashSet() {
        return Iterables.intoNewHashSet(this);
    }
}
