/*
 * Copyright (c) 2013, Victor Nazarov <asviraspossible@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Victor Nazarov nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.github.sviperll.collection.iterator;

import com.github.sviperll.BiFunction;
import com.github.sviperll.collection.FluentIterator;
import com.github.sviperll.collection.Iterators;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import com.github.sviperll.Function;
import com.github.sviperll.Predicate;

public class IteratorFluentIterator<T> implements FluentIterator<T> {
    private final Iterator<T> iterator;

    public IteratorFluentIterator(Iterator<T> iterator) {
        this.iterator = iterator;
    }

    @Override
    public boolean hasNext() {
        return iterator.hasNext();
    }

    @Override
    public T next() {
        return iterator.next();
    }

    @Override
    public void remove() {
        iterator.remove();
    }

    @Override
    public <U> FluentIterator<U> map(Function<? super T, ? extends U> function) {
        return Iterators.map(this, function);
    }

    @Override
    public FluentIterator<T> filter(Predicate<? super T> predicate) {
        return Iterators.filter(this, predicate);
    }

    @Override
    public FluentIterator<T> take(int count) {
        return Iterators.take(this, count);
    }

    @Override
    public FluentIterator<T> drop(int count) {
        return Iterators.drop(this, count);
    }

    @Override
    public <R> R fold(R seed, BiFunction<? super T, R, R> combiner) {
        return Iterators.fold(this, seed, combiner);
    }

    @Override
    public FluentIterator<T> append(Iterator<? extends T> iterator) {
        return Iterators.append(this, iterator);
    }

    @Override
    public List<T> intoNewArrayList() {
        return Iterators.intoNewArrayList(this);
    }

    @Override
    public Set<T> intoNewTreeSet() {
        return Iterators.intoNewTreeSet(this);
    }

    @Override
    public Set<T> intoNewHashSet() {
        return Iterators.intoNewHashSet(this);
    }
}
